#!/usr/bin/wish

proc Scrolled_Canvas { life args } {
    toplevel $life

    wm title $life "TkLife Canvas"
    wm protocol $life WM_DELETE_WINDOW "destroy $life"

    eval {canvas $life.canvas -xscrollcommand [list $life.xscroll set] \
	    -yscrollcommand [list $life.yscroll set] \
	    -highlightthickness 0 -borderwidth 0} $args
    
    scrollbar $life.xscroll -orient horizontal \
	    -command [list $life.canvas xview]
    scrollbar $life.yscroll -orient vertical \
	    -command [list $life.canvas yview]

    grid $life.canvas $life.yscroll -sticky news
    grid $life.xscroll -sticky ew
    grid rowconfigure $life 0 -weight 1
    grid columnconfigure $life 0 -weight 1
    return $life.canvas
}

proc Life_Begin {} {

    global mod_value
    global current_iteration
    global number_of_iterations
    global run_status
    global x_n
    global y_n
    global z_n
    global action_status

    set number_of_iterations [ expr $x_n*$y_n*1000 ]

    while { $current_iteration <= $number_of_iterations && $action_status == "RUN"} {

	if {$run_status == "FAST"} {
	    set update_interval 0
	}
	if {$run_status == "SLOW"} {
	    set update_interval 200
	}

	after $update_interval
	update

	set x [expr rand()]
	set y [expr rand()]

	set x [expr $x*$x_n]
	set y [expr $y*$y_n]

	set xi [expr round($x)]
	set yi [expr round($y)]

	set r [expr rand()*15]
	set g [expr rand()*15]
	set b [expr rand()*15]

	set ri [expr round($r)]
	set gi [expr round($g)]
	set bi [expr round($b)]

	set rgb_R [format "#%01x" $ri]
	set rgb_G [format "#%01x" $gi]
	set rgb_B [format "#%01x" $bi]

	regsub -all "\#" $rgb_R "" rgb_R
	regsub -all "\#" $rgb_G "" rgb_G
	regsub -all "\#" $rgb_B "" rgb_B

	if { $rgb_R == "0" || $rgb_R == "1" || $rgb_R == "2" || $rgb_R == "3" } {
		set rgb_R "0"
	}
	if { $rgb_R == "4" || $rgb_R == "5" || $rgb_R == "6" || $rgb_R == "7" } {
                set rgb_R "5"
        }
	if { $rgb_R == "8" || $rgb_R == "9" || $rgb_R == "a" || $rgb_R == "b" } {
                set rgb_R "a"
        }
	if { $rgb_R == "c" || $rgb_R == "d" || $rgb_R == "e" || $rgb_R == "f" } {
                set rgb_R "f"
        }

	if { $rgb_G == "0" || $rgb_G == "1" || $rgb_G == "2" || $rgb_G == "3" } {
                set rgb_G "0"
        }
        if { $rgb_G == "4" || $rgb_G == "5" || $rgb_G == "6" || $rgb_G == "7" } {
                set rgb_G "5"
        }
        if { $rgb_G == "8" || $rgb_G == "9" || $rgb_G == "a" || $rgb_G == "b" } {
                set rgb_G "a"
        }
        if { $rgb_G == "c" || $rgb_G == "d" || $rgb_G == "e" || $rgb_G == "f" } {
                set rgb_G "f"
        }

	if { $rgb_B == "0" || $rgb_B == "1" || $rgb_B == "2" || $rgb_B == "3" } {
                set rgb_B "0"
        }
        if { $rgb_B == "4" || $rgb_B == "5" || $rgb_B == "6" || $rgb_B == "7" } {
                set rgb_B "5"
        }
        if { $rgb_B == "8" || $rgb_B == "9" || $rgb_B == "a" || $rgb_B == "b" } {
                set rgb_B "a"
        }
        if { $rgb_B == "c" || $rgb_B == "d" || $rgb_B == "e" || $rgb_B == "f" } {
                set rgb_B "f"
        }

	set rgb "#$rgb_R$rgb_G$rgb_B"

	update

	set s [expr fmod($current_iteration,$mod_value)]

	.current_rgb_value config -text "0 - 0 - 0"
	.current_rgb_color config -bg "black" -width 8

	if { $xi != 0 && $yi != 0 && $s == 0 } {
	    .current_rgb_value config -text "$rgb_R - $rgb_G - $rgb_B"
	    .current_rgb_color config -bg $rgb -width 8
	    Metamorphic_Node1 $xi $yi $rgb
	}

	if { $xi > 1 && $yi > 1 && $s != 0 } {
	    if { $xi < $x_n && $yi < $y_n } {
		Metamorphic_Node2 $xi $yi
	    }
	}

	incr current_iteration
    }
    bell
}

proc Metamorphic_Node1 { xi yi rgb } {

    global node_array
    .life.canvas itemconfigure $node_array($xi,$yi) -fill $rgb -outline $rgb
    set tag_list [.life.canvas gettags $node_array($xi,$yi)]
    .life.canvas addtag "painted" withtag $tag_list

}

proc Metamorphic_Node2 { xi yi } {

    global node_array

    set xl [expr $xi - 1]
    set xr [expr $xi + 1]
    set yu [expr $yi - 1]
    set yd [expr $yi + 1]

    set inner_node $node_array($xi,$yi)
    set upper_node $node_array($xi,$yu)
    set lower_node $node_array($xi,$yd)
    set left_node  $node_array($xl,$yi)
    set right_node $node_array($xr,$yi)

    set left_up    $node_array($xl,$yu)
    set left_down  $node_array($xl,$yd)
    set right_up   $node_array($xr,$yu)
    set right_down $node_array($xr,$yd)

    ### GET COLOR INFO FOR NODES ###

    set tag_list_in [.life.canvas gettags $inner_node]
    set tag_list_up [.life.canvas gettags $upper_node]
    set tag_list_lw [.life.canvas gettags $lower_node]
    set tag_list_lf [.life.canvas gettags  $left_node]
    set tag_list_rt [.life.canvas gettags $right_node]

    set color_in [lindex [.life.canvas itemconfig $inner_node -fill] 4]
    set color_up [lindex [.life.canvas itemconfig $upper_node -fill] 4]
    set color_lw [lindex [.life.canvas itemconfig $lower_node -fill] 4]
    set color_lf [lindex [.life.canvas itemconfig  $left_node -fill] 4]
    set color_rt [lindex [.life.canvas itemconfig $right_node -fill] 4]

    ### END OF COLOR INFO ###

    ### CONDITION A ###
    if { [lindex $tag_list_in 1] == "painted" && [lindex $tag_list_up 1] == "painted" &&  [lindex $tag_list_lw 1] == "painted" && [lindex $tag_list_lf 1] == "painted" && [lindex $tag_list_rt 1] == "painted"} {
	### CONDITION B ###
	if { $color_in == $color_up && $color_in == $color_lw && $color_in ==  $color_lf && $color_in ==  $color_rt } {
	    .life.canvas itemconfigure $left_up    -fill $color_in -outline $color_in
	    .life.canvas itemconfigure $left_down  -fill $color_in -outline $color_in
	    .life.canvas itemconfigure $right_up   -fill $color_in -outline $color_in
	    .life.canvas itemconfigure $right_down -fill $color_in -outline $color_in
	    update
	} else {
	.life.canvas itemconfigure $inner_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure $upper_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure $lower_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure  $left_node -fill $color_in -outline $color_in
	.life.canvas itemconfigure $right_node -fill $color_in -outline $color_in
	update
	}
    }
}

proc Life_Plot {} {

    global x_n
    global y_n
    global z_n

    set  l 1
    set y1 5

    while { $l <= $y_n } {
	Plot_X $l $y1
	incr l
	set y1 [expr $y1 + $z_n + 2]
    }
    Life_Begin
}

proc Plot_X { l y1 } {

    global node_array
    global x_n
    global y_n
    global z_n

    set  k 1
    set x1 5
    while { $k <= $x_n } {
	set x2 [expr $x1 + $z_n]
	set y2 [expr $y1 + $z_n]
	set node_tag "node_$l\_$k"
	set node_array($k,$l) [.life.canvas create rect $x1 $y1 $x2 $y2 \
		-fill "#000" -outline "#fff" -tag "$node_tag"]
	update
	incr k
	set x1 [expr $x1 + $z_n + 2]
    }
}

proc Get_Parameters { argv } {

    global x_n
    global y_n
    global z_n

    puts "$argv PARAMETERS"

    if { [lindex $argv 0] != "" } {
	set x_n [lindex $argv 0]
    } else {
	set x_n 100
    }
    if { [lindex $argv 1] != "" } {
	set y_n [lindex $argv 1]
    } else {
	set y_n 75
    }
    if { [lindex $argv 2] != "" } {
	set z_n [lindex $argv 2]
    } else {
	set z_n 7
    }

    puts "X: $x_n  Y: $y_n  Z: $z_n"

    set scrollmax_x [expr ($x_n*$z_n + 20)]
    set scrollmax_y [expr ($y_n*$z_n + 20)]
    Scrolled_Canvas .life -width 700 -height 550 -bg black \
	    -scrollregion "0 0 $scrollmax_x $scrollmax_y"

    Life_Plot
}

proc Create_64_Palette {} {

    .first_layer create rect  0  0  30  30 -fill "#000" -tag "0_0_0"
    .first_layer create rect 30  0  60  30 -fill "#005" -tag "0_0_5"
    .first_layer create rect 60  0  90  30 -fill "#00a" -tag "0_0_a"
    .first_layer create rect 90  0 120  30 -fill "#00f" -tag "0_0_f"

    .first_layer create rect  0 30  30  60 -fill "#050" -tag "0_5_0"
    .first_layer create rect 30 30  60  60 -fill "#055" -tag "0_5_5"
    .first_layer create rect 60 30  90  60 -fill "#05a" -tag "0_5_a"
    .first_layer create rect 90 30 120  60 -fill "#05f" -tag "0_5_f"

    .first_layer create rect  0 60  30  90 -fill "#0a0" -tag "0_a_0"
    .first_layer create rect 30 60  60  90 -fill "#0a5" -tag "0_a_5"
    .first_layer create rect 60 60  90  90 -fill "#0aa" -tag "0_a_a"
    .first_layer create rect 90 60 120  90 -fill "#0af" -tag "0_a_f"

    .first_layer create rect  0 90  30 120 -fill "#0f0" -tag "0_f_0"
    .first_layer create rect 30 90  60 120 -fill "#0f5" -tag "0_f_5"
    .first_layer create rect 60 90  90 120 -fill "#0fa" -tag "0_f_a"
    .first_layer create rect 90 90 120 120 -fill "#0ff" -tag "0_f_f"
    ############
    .second_layer create rect  0  0  30  30 -fill "#500" -tag "5_0_0"
    .second_layer create rect 30  0  60  30 -fill "#505" -tag "5_0_5"
    .second_layer create rect 60  0  90  30 -fill "#50a" -tag "5_0_a"
    .second_layer create rect 90  0 120  30 -fill "#50f" -tag "5_0_f"

    .second_layer create rect  0 30  30  60 -fill "#550" -tag "5_5_0"
    .second_layer create rect 30 30  60  60 -fill "#555" -tag "5_5_5"
    .second_layer create rect 60 30  90  60 -fill "#55a" -tag "5_5_a"
    .second_layer create rect 90 30 120  60 -fill "#55f" -tag "5_5_f"

    .second_layer create rect  0 60  30  90 -fill "#5a0" -tag "5_a_0"
    .second_layer create rect 30 60  60  90 -fill "#5a5" -tag "5_a_5"
    .second_layer create rect 60 60  90  90 -fill "#5aa" -tag "5_a_a"
    .second_layer create rect 90 60 120  90 -fill "#5af" -tag "5_a_f"

    .second_layer create rect  0 90  30 120 -fill "#5f0" -tag "5_f_0"
    .second_layer create rect 30 90  60 120 -fill "#5f5" -tag "5_f_5"
    .second_layer create rect 60 90  90 120 -fill "#5fa" -tag "5_f_a"
    .second_layer create rect 90 90 120 120 -fill "#5ff" -tag "5_f_f"
    ############
    .third_layer create rect  0  0  30  30 -fill "#a00" -tag "a_0_0"
    .third_layer create rect 30  0  60  30 -fill "#a05" -tag "a_0_5"
    .third_layer create rect 60  0  90  30 -fill "#a0a" -tag "a_0_a"
    .third_layer create rect 90  0 120  30 -fill "#a0f" -tag "a_0_f"

    .third_layer create rect  0 30  30  60 -fill "#a50" -tag "a_5_0"
    .third_layer create rect 30 30  60  60 -fill "#a55" -tag "a_5_5"
    .third_layer create rect 60 30  90  60 -fill "#a5a" -tag "a_5_a"
    .third_layer create rect 90 30 120  60 -fill "#a5f" -tag "a_5_f"

    .third_layer create rect  0 60  30  90 -fill "#aa0" -tag "a_a_0"
    .third_layer create rect 30 60  60  90 -fill "#aa5" -tag "a_a_5"
    .third_layer create rect 60 60  90  90 -fill "#aaa" -tag "a_a_a"
    .third_layer create rect 90 60 120  90 -fill "#aaf" -tag "a_a_f"

    .third_layer create rect  0 90  30 120 -fill "#af0" -tag "a_f_0"
    .third_layer create rect 30 90  60 120 -fill "#af5" -tag "a_f_5"
    .third_layer create rect 60 90  90 120 -fill "#afa" -tag "a_f_a"
    .third_layer create rect 90 90 120 120 -fill "#aff" -tag "a_f_f"
    ############
    .fourth_layer create rect  0  0  30  30 -fill "#f00" -tag "f_0_0"
    .fourth_layer create rect 30  0  60  30 -fill "#f05" -tag "f_0_5"
    .fourth_layer create rect 60  0  90  30 -fill "#f0a" -tag "f_0_a"
    .fourth_layer create rect 90  0 120  30 -fill "#f0f" -tag "f_0_f"

    .fourth_layer create rect  0 30  30  60 -fill "#f50" -tag "f_5_0"
    .fourth_layer create rect 30 30  60  60 -fill "#f55" -tag "f_5_5"
    .fourth_layer create rect 60 30  90  60 -fill "#f5a" -tag "f_5_a"
    .fourth_layer create rect 90 30 120  60 -fill "#f5f" -tag "f_5_f"

    .fourth_layer create rect  0 60  30  90 -fill "#fa0" -tag "f_a_0"
    .fourth_layer create rect 30 60  60  90 -fill "#fa5" -tag "f_a_5"
    .fourth_layer create rect 60 60  90  90 -fill "#faa" -tag "f_a_a"
    .fourth_layer create rect 90 60 120  90 -fill "#faf" -tag "f_a_f"

    .fourth_layer create rect  0 90  30 120 -fill "#ff0" -tag "f_f_0"
    .fourth_layer create rect 30 90  60 120 -fill "#ff5" -tag "f_f_5"
    .fourth_layer create rect 60 90  90 120 -fill "#ffa" -tag "f_f_a"
    .fourth_layer create rect 90 90 120 120 -fill "#fff" -tag "f_f_f"
}

proc Stop_Action {} {
    global action_status
    set action_status "STOP"
}

proc Cont_Action {} {
    global action_status
    set action_status "RUN"
    Life_Begin
}

#############################
#                           #
#        MAIN BODY          #
#                           #
#############################

set introduction_text "

################################################
#                                              #
#   Default parameters:                        #
#   x_n - number of elements on X axis (100)   #
#   y_n - number of elements on Y axis  (75)   #
#   z_n - size of elements in pixels     (7)   #
#                                              #
#   You can override default parameters by     #
#   running the program with three arguments,  #
#   for example:                               #
#                                              #
#   TkLife.tcl 150 100 5                       #
#                                              #
################################################

"
puts $introduction_text
bell
after 3000

################
#              #
#     GUI      #
#              #
################

wm title . "TkLife"
wm protocol . WM_DELETE_WINDOW "destroy ."
. config -bg black

global mod_value
global current_iteration
global number_of_iterations
global run_status
global rgb_value
global rgb_color
global action_status

set mod_value 15
set current_iteration 0
set run_status "FAST"
set rgb_value  "0 - 0 - 0"
set rgb_color  "#000"
set action_status "RUN"

label .curr_iteration -text "Current Iteration: " -bg "#003" -fg "#fff"
entry .curr_iteration_entry -relief sunken -textvariable current_iteration \
	-bg "#003" -fg "#fff" -width 16 -insertbackground lightyellow
grid .curr_iteration .curr_iteration_entry

label .numb_iteration -text "Number of Iterations: " -bg "#003" -fg "#fff"
entry .numb_iteration_entry -relief sunken -textvariable number_of_iterations \
	-bg "#003" -fg "#fff" -width 16 -insertbackground lightyellow
grid .numb_iteration .numb_iteration_entry

label .mut_value -text "Mutation K Value: " -bg "#003" -fg "#fff"
entry .mut_value_entry -relief sunken -textvariable mod_value \
	-bg "#003" -fg "#fff" -width 16 -insertbackground lightyellow
grid .mut_value .mut_value_entry

label .current_rgb_value_t -text "RGB value" -bg "#003" -fg "#fff" -width 12
label .current_rgb_color_t -text "current color" -bg "#003" -fg "#fff" -width 12

grid .current_rgb_value_t .current_rgb_color_t

label .current_rgb_value -text $rgb_value -bg "#003" -fg "#ffa" -width 12
label .current_rgb_color -text "" -bg $rgb_color
grid .current_rgb_value .current_rgb_color

label .layer_1 -text "layer 1" -bg "#003" -fg "#fff" -width 12
label .layer_2 -text "layer 2" -bg "#003" -fg "#fff" -width 12

grid .layer_1 .layer_2

canvas .first_layer -width 120 -height 120
canvas .second_layer -width 120 -height 120
grid .first_layer .second_layer -padx 5 -pady 5

label .layer_3 -text "layer 3" -bg "#003" -fg "#fff" -width 12
label .layer_4 -text "layer 4" -bg "#003" -fg "#fff" -width 12

grid .layer_3 .layer_4

canvas .third_layer -width 120 -height 120
canvas .fourth_layer -width 120 -height 120
grid .third_layer .fourth_layer -padx 5 -pady 5

radiobutton .run_value1 -relief sunken -variable run_status \
    -text " FAST " -value "FAST" -bg "#003" -fg gray80 -width 8
radiobutton .run_value2 -relief sunken -variable  run_status\
    -text " SLOW " -value "SLOW" -bg "#003" -fg gray80 -width 8
grid .run_value1 .run_value2 -padx 5 -pady 5

button .stop_button -bg "#003" -fg gray80 -text "Stop" -command {Stop_Action} -width 8
button .cont_button -bg "#003" -fg gray80 -text "Continue" -command {Cont_Action} -width 8

grid .stop_button .cont_button

Create_64_Palette

Get_Parameters $argv

####################
#                  #
#     THE END      #
#                  #
####################
